#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
All bot classes

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 13:54
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from ._smart import TextBot, MarkdownBot, ImageBot
from ._smart import NewsBot, FileBot, VoiceBot
from ._smart import TextCardBot, NewsCardBot, SmartBot
from ._smart import MarkdownBotV2

__all__ = [
    "TextBot",
    "MarkdownBot",
    "MarkdownBotV2",
    "ImageBot",
    "NewsBot",
    "FileBot",
    "VoiceBot",
    "TextCardBot",
    "NewsCardBot",
    "SmartBot",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Main Package entrance.

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 13:45
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
News Card type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/4 17:21
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from functools import partial
from logging import getLogger
from typing import List

from jmespath import search

from . import TemplateCardKeys, TemplateCardRequirements
from .._abstract import ConvertedData, AbstractBot

logger = getLogger(__name__)


class NewsCardBot(AbstractBot):
    """News Card type message Wecom Group Bot"""

    _VALID_KEYS: List[str] = TemplateCardKeys + [
        "card_image", "image_text_area", "vertical_content_list"
    ]

    @property
    def _doc_key(self) -> str:
        return "图文展示模版卡片"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        """
        reqs = {
            "Both `main_title` and `main_title.title` must be existed":
                partial(
                    search, """
                    main_title == null ||
                    main_title.title == null ||
                    main_title.title == ''
                """),
            "Both `card_image` and `card_image.url` must be existed":
                partial(
                    search, """
                    card_image == null ||
                    card_image.url == null ||
                    card_image.url == ''
                """),
            "When `image_text_area.type` is 1, the `image_text_area.url` must be existed":
                partial(
                    search, """
                    image_text_area.type == `1` &&
                    (image_text_area.url == null || image_text_area.url == '')
                """),
            "When `image_text_area.type` is 2, the `image_text_area.appid` must be existed":
                partial(
                    search, """
                    image_text_area.type == `2` &&
                    (image_text_area.appid == null || image_text_area.appid == '')
                """),
            "The `image_text_area.image_url` must be existed":
                partial(
                    search, """
                    image_text_area != null &&
                    (image_text_area.image_url == null || image_text_area.image_url == '')
                """),
            "The `vertical_content_list.title` must be existed":
                partial(
                    search, """
                    length(
                        (vertical_content_list || `[]`)[?
                            title == null || title == ''
                        ]
                    ) != `0`
                """),
            **TemplateCardRequirements
        }
        for msg, cmd in reqs.items():
            logger.debug("Validating parameter error: %s", msg)
            if cmd(kwargs):
                logger.critical("[NO PASS] Parameter validation error: %s", msg)
                raise ValueError(msg)

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to text card format data.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted data.
        """
        kw_ = {
            key: val for key, val in kwargs.items() if key in self._VALID_KEYS
        }
        result = ({
            "msgtype": "template_card",
            "template_card": {
                "card_type": "news_notice",
                **kw_,
            }
        },)
        return result, kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Test Smart bot.

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/6 14:40
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from logging import DEBUG, basicConfig
from pathlib import Path
from random import randint
from urllib.parse import urlparse, unquote

from dotenv import load_dotenv

# pylint: disable=import-error
from src.pywgb import SmartBot
from tests.test_main import TEST_VALID_ARTICLES, TEST_VALID_MARKDOWN_V2
from tests.test_main import TEST_VALID_MARKDOWN
from tests.test_main import TEST_VALID_NEWS_CARD
from tests.test_main import TEST_VALID_TEXT_CARD
from tests.test_main import VALID_KEY, env_file

basicConfig(level=DEBUG, format="%(levelname)s %(name)s %(lineno)d %(message)s")
load_dotenv(env_file, override=True)


def test_initial() -> None:
    """
    Test SmartBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = SmartBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_send() -> None:
    """
    Test basic send.
    :return:
    """
    text = f"This is a test TEXT message: {randint(1, 100)}"
    image = Path(__file__).with_name("test.png")
    file = Path(__file__).with_name("test.txt")
    voice = Path(__file__).with_name("test.amr")
    tests = {
        "text": ((text,), {}),
        "text_with_mention": ((text,), {
            "mentioned_list": ["@all"]
        }),
        "markdown": ((TEST_VALID_MARKDOWN,), {}),
        "markdown_v2": ((TEST_VALID_MARKDOWN_V2,), {}),
        "news": ((), {
            "articles": TEST_VALID_ARTICLES
        }),
        "image": ((), {
            "file_path": image
        }),
        "file": ((), {
            "file_path": file
        }),
        "voice": ((), {
            "file_path": voice
        }),
        "text_card": ((), {
            **TEST_VALID_TEXT_CARD
        }),
        "news_card": ((), {
            **TEST_VALID_NEWS_CARD
        }),
    }
    bot = SmartBot(VALID_KEY)
    print(bot)
    for type_, (args, kwargs) in tests.items():
        print("Testing:", type_)
        result = bot.send(*args, **kwargs)
        assert result["errcode"] == 0



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
News type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/30 10:31
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from ._abstract import ConvertedData, AbstractBot


class NewsBot(AbstractBot):
    """News type message Wecom Group Bot"""

    @property
    def _doc_key(self) -> str:
        return "图文类型"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        """
        try:
            articles = kwargs["articles"]
        except KeyError as error:
            raise ValueError("The articles parameter is required.") from error
        if not articles:
            raise ValueError("The articles parameter is empty.")
        if len(articles) > 8:
            raise ValueError("Too many articles. The maximum limit is 8")
        # Check the article's required parameters
        test = kwargs.get("test")
        for index, article in enumerate(articles):
            if not isinstance(article, dict) or test == "article_data_error":
                msg_ = f"The No.{index + 1} article data is not a dict"
                raise ValueError(msg_)
            for param in ["title", "url"]:
                if param not in article or not article[
                        param] or test == "article_parameter_error":
                    msg_ = f"The No.{index + 1} article lack required parameter: {param}"
                    raise ValueError(msg_)

    # pylint:disable=unused-argument
    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to News format.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted message.
        """
        result = ({
            "msgtype": "news",
            "news": {
                "articles": kwargs["articles"]
            }
        },)
        return result, kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
File type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/30 14:40
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from ._abstract import ConvertedData, AbstractBot
from .._deco import verify_file


class FileBot(AbstractBot):
    """File type message Wecom Group Bot"""

    @property
    def _doc_key(self) -> str:
        return "文件类型"

    @verify_file
    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        """

    # pylint:disable=unused-argument
    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to File format.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted message.
        """
        file_path = kwargs["file_path"]
        # Check file size, only smaller than `20M` and large than `5B`
        with open(file_path, "rb") as _:
            content = _.read()
        size_range = 5 < len(content) < 20 * pow(1024, 2)
        if not size_range or kwargs.get("test") == "oversize_file":
            raise ValueError("The file size is out of range: 5B < SIZE < 20M")
        media_id = self.upload(file_path)
        result = {"msgtype": "file", "file": {"media_id": media_id}}
        return (result,), kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Wecom(A.K.A. WeChat Work) Group Bot python API.


- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 13:42
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

__author__ = 'Rex Zhou'
__copyright__ = 'Copyright © 2025 Rex Zhou. All rights reserved.'
__credits__ = [__author__]
__license__ = "MIT"
__maintainer__ = __author__
__email__ = '879582094@qq.com'

from importlib.metadata import version

from . import bot
from .bot import SmartBot

__version__ = version('pywgb')
__all__ = ["bot", "SmartBot"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Image type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/29 14:05
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from base64 import b64encode
from hashlib import md5
from pathlib import Path

from ._abstract import ConvertedData, AbstractBot
from .._deco import verify_file


class ImageBot(AbstractBot):
    """Image type message Wecom Group Bot"""

    @property
    def _doc_key(self) -> str:
        return "图片类型"

    @verify_file
    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        :raise ValueError: Verify error.
        """
        suffix = Path(kwargs["file_path"]).suffix.lower()
        test = kwargs.get("test")
        # Check format, only support: `.jpg` and `.png`
        if suffix not in [".jpg", ".png"] or test == "wrong_format_image":
            raise ValueError("Just support image type: jpg or png")

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to Image format.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted message.
        """
        file_path = Path(kwargs["file_path"])
        # Check image size, only smaller than `2M`
        max_size = 2 * pow(1024, 2)
        with open(file_path, "rb") as _:
            content = _.read()
        if len(content) > max_size or kwargs.get("test") == "oversize_image":
            raise ValueError("The image is too large, more than 2M")
        result = ({
            "msgtype": "image",
            "image": {
                "base64": b64encode(content).decode("utf-8"),
                "md5": md5(content).hexdigest(),
            }
        },)
        return result, kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Decorators module

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/29 14:48
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from functools import wraps
from logging import getLogger
from pathlib import Path
from time import sleep

from requests import RequestException

logger = getLogger(__name__)


def verify_and_convert_arguments(function):
    """
    Verify and convert the arguments to standard format.
    :param function: Callable object.
    :return: Result dict.
    """

    # pylint: disable=protected-access
    @wraps(function)
    def wrapper(self, *args, **kwargs) -> dict:
        logger.debug("---- %s ----", verify_and_convert_arguments.__name__)
        logger.debug("Positional arguments: %s", args)
        logger.debug("Other kwargs: %s", kwargs)
        try:
            self._verify_arguments(*args, **kwargs)
        except ValueError as error:
            logger.critical("Verification of arguments failed: %s", error)
            raise error
        try:
            args, kwargs = self._convert_arguments(*args, **kwargs)
        except ValueError as error:
            logger.critical("Convertion of arguments failed: %s", error)
            raise error
        logger.debug("Converted arguments: %s", args)
        logger.debug("Converted other kwargs: %s", kwargs)
        logger.debug("---- %s ----", verify_and_convert_arguments.__name__)
        return function(self, *args, **kwargs)

    return wrapper


def detect_overheat(function):
    """
    Detect overheat.
    :param function: Callable object.
    :return: Result dict.
    """
    overheat_threshold: int = 60
    overheat_error_code: int = 45009

    # pylint: disable=protected-access
    @wraps(function)
    def wrapper(self, *args, **kwargs) -> dict:
        """
        Detect overheat.
        :param self: Object instance.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: Result dict.
        """
        logger.debug("==== %s ====", detect_overheat.__name__)
        logger.debug("Positional arguments: %s", args)
        logger.debug("Other kwargs: %s", kwargs)
        if kwargs.get("test") == "overheat":
            kwargs.pop("test")
            threshold = 0
            result = {"errcode": overheat_error_code}
        else:
            threshold = overheat_threshold
            if self._OVERHEAT >= 0:
                logger.warning("Overheat detected.")
                while self._OVERHEAT >= 0:
                    print(f"\rCooling down: {self._OVERHEAT:02d}s",
                          end='',
                          flush=True)
                    sleep(1)
                    self._OVERHEAT -= 1
                print()
            result = function(self, *args, **kwargs)
        if result.get("errcode") == overheat_error_code:
            self._OVERHEAT = threshold
            result = wrapper(self, *args, **kwargs)
        logger.debug("==== %s ====", detect_overheat.__name__)
        return result

    return wrapper


def handle_request_exception(function):
    """
    Handle request exception.
    :param function: Callable object.
    :return:
    """

    @wraps(function)
    def wrapper(self, *args, **kwargs):
        """
        Handle request exception.
        :param self: Object instance.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: Result dict.
        """
        logger.debug("#### %s ####", handle_request_exception.__name__)
        logger.debug("Positional arguments: %s", args)
        logger.debug("Other kwargs: %s", kwargs)
        try:
            if (test := kwargs.get("test")) == "request_error":
                raise RequestException
            if test == "api_error":
                result = {"errcode": -1}
            else:
                result = function(self, *args, **kwargs)
            if result.get("errcode") != 0:
                msg = f"Request failed, please refer to the official manual: {self.doc}"
                logger.error(msg)
                logger.error("Error message: %s", result)
                raise IOError(msg)
        except RequestException as error:
            msg = f"Unable to initiate API request correctly: {error}"
            logger.error(msg)
            raise ConnectionRefusedError(msg) from error
        logger.debug("#### %s ####", handle_request_exception.__name__)
        return result

    return wrapper


def verify_file(function):
    """
    Verify file, include: amr / png / jpg or other type files.
    :param function: Callable function.
    :return:
    """

    @wraps(function)
    def wrapper(self, *args, **kwargs):
        logger.debug("$$$$ %s $$$$", verify_file.__name__)
        logger.debug("Positional arguments: %s", args)
        logger.debug("Other kwargs: %s", kwargs)
        file_path = kwargs.get("file_path")
        try:
            file_path = file_path if file_path else args[0]
        except IndexError as error:
            raise ValueError("The file_path parameter is required.") from error
        if not Path(file_path).exists():
            raise ValueError("The file_path parameter not exists.")
        logger.debug("$$$$ %s $$$$", verify_file.__name__)
        return function(self, *args, **kwargs)

    return wrapper



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Unit test

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/3 14:42
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from logging import basicConfig, DEBUG
from os import getenv
from pathlib import Path

from dotenv import load_dotenv

# pylint: disable=import-error
from src.pywgb import SmartBot

basicConfig(level=DEBUG, format="%(levelname)s %(name)s %(lineno)d %(message)s")
env_file = Path(__file__).parent.with_name(".env")
errors_file = Path(__file__).with_name("card_errors.yml")
load_dotenv(env_file, override=True)
VALID_KEY = getenv("VALID_KEY")
bot = SmartBot(VALID_KEY)
_COL = [
    bot.markdown_feature.green, bot.markdown_feature.gray,
    bot.markdown_feature.orange
]
_COL = "".join(_COL[idx % 3](ltr) for idx, ltr in enumerate("colorful"))
TEST_VALID_MARKDOWN = f"""
# TESTING

> Author: **Rex**

This is a {_COL} Markdown message
"""
_TABLE = [
    ["Name", "Gender", "Title"],
    ["Julia", "Female", "Accounting"],
    ["Jess", "Female", "Reception"],
    ["Tom", "Male", "Manager"],
    ["Grance", "Male", "Testing"],
    ["Rex", "Male", "DevOps"],
]
TEST_VALID_MARKDOWN_V2 = f"""
# TESTING v2

*Italics*

> Author: **Rex**
>> Version: v2
>>> Peace and love

---

- Unordered List 1
- Unordered List 2
  - Unordered List 2.1
  - Unordered List 2.2
1. Ordered List 1
2. Ordered List 2

![Picture](https://res.mail.qq.com/node/ww/wwopenmng/images/independent/doc/test_pic_msg1.png)

```
There is a test code block.
```

Here is a empty string when the table is less than 2 rows.

{bot.markdown_feature.list2table(_TABLE[:1])}

Here is a test table.

{bot.markdown_feature.list2table(_TABLE)}

"""

TEST_VALID_ARTICLES = [{
    "title":
        "中秋节礼品领取",
    "description":
        "今年中秋节公司有豪礼相送",
    "url":
        "www.qq.com",
    "picurl":
        "http://res.mail.qq.com/node/ww/wwopenmng/images/independent/doc/test_pic_msg1.png"
}]
TEST_VALID_TEXT_CARD = {
    "main_title": {
        "title": "欢迎使用企业微信",
        "desc": "您的好友正在邀请您加入企业微信"
    },
    "emphasis_content": {
        "title": "100",
        "desc": "数据含义"
    },
    "quote_area": {
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi",
        "title": "引用文本标题",
        "quote_text": "Jack：企业微信真的很好用~\nBalian：超级好的一款软件！"
    },
    "sub_title_text": "下载企业微信还能抢红包！",
    "horizontal_content_list": [{
        "keyname": "邀请人",
        "value": "张三"
    }, {
        "keyname": "企微官网",
        "value": "点击访问",
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi"
    }],
    "jump_list": [{
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi",
        "title": "企业微信官网"
    }],
    "card_action": {
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi",
    }
}
TEST_VALID_NEWS_CARD = {
    "source": {
        "icon_url":
            "https://wework.qpic.cn/wwpic/252813_jOfDHtcISzuodLa_1629280209/0",
        "desc":
            "企业微信",
        "desc_color":
            0
    },
    "main_title": {
        "title": "欢迎使用企业微信",
        "desc": "您的好友正在邀请您加入企业微信"
    },
    "card_image": {
        "url":
            "https://wework.qpic.cn/wwpic/354393_4zpkKXd7SrGMvfg_1629280616/0",
        "aspect_ratio":
            2.25
    },
    "image_text_area": {
        "type":
            1,
        "url":
            "https://work.weixin.qq.com",
        "title":
            "欢迎使用企业微信",
        "desc":
            "您的好友正在邀请您加入企业微信",
        "image_url":
            "https://wework.qpic.cn/wwpic/354393_4zpkKXd7SrGMvfg_1629280616/0"
    },
    "quote_area": {
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi",
        "appid": "APPID",
        "pagepath": "PAGEPATH",
        "title": "引用文本标题",
        "quote_text": "Jack：企业微信真的很好用~\nBalian：超级好的一款软件！"
    },
    "vertical_content_list": [{
        "title": "惊喜红包等你来拿",
        "desc": "下载企业微信还能抢红包！"
    }],
    "horizontal_content_list": [{
        "keyname": "邀请人",
        "value": "张三"
    }, {
        "keyname": "企微官网",
        "value": "点击访问",
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi"
    }],
    "jump_list": [{
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi",
        "title": "企业微信官网"
    }],
    "card_action": {
        "type": 1,
        "url": "https://work.weixin.qq.com/?from=openApi"
    }
}


# pylint: disable=protected-access
def main():  # pragma: no cover
    """
    For unit testing
    :return:
    """
    result = bot.send(TEST_VALID_MARKDOWN)
    print(result)


if __name__ == "__main__":  # pragma: no cover
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Abstract bot classes

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/6 09:01
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
__all__ = ["AbstractBot", "FilePathLike", "ConvertedData"]

from abc import ABC, abstractmethod
from functools import partial
from logging import getLogger
from os import PathLike
from pathlib import Path
from typing import Union, Tuple, Dict, List
from urllib.parse import urlparse, parse_qs, urljoin, quote
from uuid import UUID

from requests import Session, session

from .._deco import handle_request_exception, verify_file
from .._deco import detect_overheat, verify_and_convert_arguments

logger = getLogger(__name__)
FilePathLike = Union[str, PathLike]
ConvertedData = Tuple[Tuple[Dict[str, str]], Dict[str, str]]


class _Basic(ABC):
    """Private bot basic class"""

    # The base path of the document
    _DOC_URL: str = "https://developer.work.weixin.qq.com/document/path/99110"
    # API Endpoint base url
    _API_BASE_URL: str = 'https://qyapi.weixin.qq.com'

    def __init__(self, key: str) -> None:
        """
        Initialize the class.
        :param key: The key of group bot webhook url.
        """
        if key is None:
            raise ValueError("Key is required")
        self.key = self._verify_uuid(key.strip())
        self._session: Session = session()

    @staticmethod
    def _parse_webhook_url(url: str) -> str:
        """
        Parse webhook url into key string.
        :param url: Webhook url.
        :return: Key string.
        """
        # If the key passed by url, split and parse it.
        try:
            query = urlparse(url).query
            params = parse_qs(query)
            if "key" not in params or not params["key"]:
                raise ValueError("Missing 'key' parameter in URL")
            return params["key"][0]
        except Exception as error:
            msg = f"Invalid webhook URL {url}."
            logger.critical(msg)
            raise ValueError(msg) from error

    def _verify_uuid(self, key: Union[str, UUID], max_attempts: int = 2) -> str:
        """
        Verify the key weather is UUID format.
        :param key: Key string
        :param max_attempts: Max number of attempts.
        :return: Result bool
        """
        if max_attempts <= 0:  # pragma: no cover
            raise ValueError("Maximum verification attempts exceeded")
        # The standard key format is UUID format.
        if isinstance(key, UUID):  # pragma: no cover
            return str(key)
        try:
            UUID(key)
            return key
        except (ValueError, TypeError, AttributeError) as error:
            try:
                key = self._parse_webhook_url(key)
                return self._verify_uuid(key, max_attempts - 1)
            except ValueError:
                ...
            raise ValueError(f"Invalid key format: {key}") from error

    def __repr__(self) -> str:
        """
        Return the class name.
        :return: Class name.
        """
        return f"{self.__class__.__name__}({self.key})"

    @property
    def _api_url(self) -> str:
        """
        Returns the address of the spliced endpoint url.
        :return: Endpoint url
        """
        end_point = urljoin(self._API_BASE_URL, self._api_path)
        return end_point

    @property
    @abstractmethod
    def _api_path(self) -> str:
        """
        The path of the API Endpoint.
        :return: path of the API Endpoint.
        """

    @property
    @abstractmethod
    def _doc_key(self) -> str:
        """
        The key of the document description.
        :return: key of the document description
        """

    @property
    def doc(self) -> str:
        """
        API URL of the document description
        :return: URL of the document
        """
        url = f"{self._DOC_URL}#{quote(self._doc_key)}"
        return url


class _MediaUploader(_Basic):
    """Upload voice and file class"""

    @property
    def _api_path(self) -> str:
        """
        The path of the API Endpoint.
        :return: path of the API Endpoint.
        """
        return "cgi-bin/webhook/upload_media"

    @property
    def _doc_key(self) -> str:
        """
        The key of the document description.
        :return: key of the document description
        """
        return "文件上传接口"

    @handle_request_exception
    def _upload_temporary_file(self, file_path: Path, file_type: str) -> dict:
        """
        Upload temporary file from file path.
        :param file_path: File path.
        :param file_type: File type.
        :return:
        """
        kwargs = {
            "url": self._api_url,
            "params": {
                "key": self.key,
                "type": file_type
            }
        }
        cmd = partial(self._session.post, **kwargs)
        with open(file_path, "rb") as files:
            response = cmd(files={"media": files})
        result = response.json()
        return result

    @verify_file
    def upload(self, file_path: FilePathLike, /, **kwargs) -> str:
        """
        Upload voice and file.
        :param file_path: The path of the file.
        :param kwargs: Other keyword arguments.
        :return: Result dict.
        """
        file_path: Path = Path(file_path)
        # Only support `AMR` format for voice, others must be file type
        file_type: str = "voice" if file_path.suffix == ".amr" else "file"
        logger.debug("~~~~ %s ~~~~", self.upload.__name__)
        logger.debug("File path: %s", file_path)
        logger.debug("Other kwargs: %s", kwargs)
        logger.debug("API endpoint URL: %s", self._api_url)
        result = self._upload_temporary_file(file_path, file_type)
        logger.debug("~~~~ %s ~~~~", self.upload.__name__)
        logger.info("%s has been uploaded: %s", file_type.capitalize(), result)
        return result["media_id"]


class AbstractBot(_Basic, ABC):
    """Abstract class of Wecom group bot."""

    # Default requests headers
    _HEADERS: dict = {"Content-Type": "application/json"}
    # Class level variable for detect overheat
    _OVERHEAT: int = -1

    def __init__(self, key: str) -> None:
        _Basic.__init__(self, key)
        ABC.__init__(self)
        self._session.headers = self._HEADERS
        self._uploader: _MediaUploader = _MediaUploader(key)

    @property
    def _api_path(self) -> str:
        """
        The path of the API Endpoint.
        :return: path of the API Endpoint.
        """
        return "cgi-bin/webhook/send"

    @property
    @abstractmethod
    def _doc_key(self) -> str:
        """
        The key of the document description.
        :return: key of the document description
        """

    # pylint:disable=unused-argument
    @handle_request_exception
    @detect_overheat
    @verify_and_convert_arguments
    def send(self,
             msg: str = None,
             /,
             articles: List[Dict[str, str]] = None,
             file_path: FilePathLike = None,
             **kwargs) -> dict:
        """
        Method of sending a message. `Refer`_

        .. _`Refer`: https://developer.work.weixin.qq.com/document/path/91770

        :param msg: Message body.
        :param articles: List of articles. Used for send news.
        :param file_path: File path. Used for send image/voice/file.
        :return: Result dict.
        """
        logger.debug("~~~~ %s ~~~~", self.send.__name__)
        logger.debug("Message: %s", msg)
        logger.debug("Articles: %s", articles)
        logger.debug("File path: %s", file_path)
        logger.debug("Other kwargs: %s", kwargs)
        logger.debug("API endpoint URL: %s", self._api_url)
        response = self._session.post(self._api_url,
                                      params={"key": self.key},
                                      json=msg)
        result = response.json()
        logger.debug("~~~~ %s ~~~~", self.send.__name__)
        logger.info("Message has been sent: %s", result)
        return result

    def upload(self, file_path: FilePathLike) -> str:
        """
        Upload temporary media file
        :param file_path: The path of the file to upload.
        :return:
        """
        result = self._uploader.upload(file_path)
        return result

    @abstractmethod
    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify arguments methods. Subclasses must complete specific implementations
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: None
        """

    @abstractmethod
    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Prepare data methods, subclasses must complete specific implementations
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Result dict.
        """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Text type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 14:53
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from ._abstract import ConvertedData, AbstractBot


def check_args(*args, maximum: int = 2048) -> None:
    """
    Check args, raise exception if invalid.
    :param args: Positional arguments.
    :param maximum: Maximum length of content.
    :return:
    """
    try:
        msg = args[0]
    except IndexError as error:
        raise ValueError("The msg parameter is required.") from error
    if not msg:
        raise ValueError("Can't send empty message.")
    # Check then message length
    if len(str(msg).encode("utf-8")) > maximum:
        raise ValueError(f"The msg parameter is too long (>{max} bytes).")


class TextBot(AbstractBot):
    """Text type message Wecom Group Bot"""

    _OPTIONAL_ARGS = ["mentioned_list", "mentioned_mobile_list"]
    _MAX_LENGTH = 2048

    @property
    def _doc_key(self) -> str:
        return "文本类型"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        """
        check_args(*args, maximum=self._MAX_LENGTH)
        for optional in self._OPTIONAL_ARGS:
            if optional not in kwargs:
                continue
            err_msg = f"The {optional} parameter should be a list of strings."
            if not isinstance(data := kwargs[optional], list):
                raise ValueError(err_msg)
            if not all(isinstance(_, str) for _ in data):
                raise ValueError(err_msg)

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to text format data.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted data.
        """
        mentioned_list: list = kwargs.get("mentioned_list", [])
        mentioned_mobile_list: list = kwargs.get("mentioned_mobile_list", [])
        result = ({
            "msgtype": "text",
            "text": {
                "content": args[0].strip(),
                "mentioned_list": mentioned_list,
                "mentioned_mobile_list": mentioned_mobile_list
            }
        },)
        return result, kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Markdown type message sender


- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 15:12
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from enum import Enum
from typing import Union, List

from ._abstract import ConvertedData, AbstractBot
from .text import check_args


class MarkdownBot(AbstractBot):
    """Markdown type message Wecom Group Bot"""

    _MAX_LENGTH: int = 4096

    class _Color(Enum):
        """Markdown _color enum"""
        INFO = "green"
        COMMENT = "gray"
        WARNING = "orange"

        @classmethod
        def get_valid_colors(cls):
            """Return list of valid colors"""
            return [_.value for _ in cls]

        @classmethod
        def get_valid_codes(cls):
            """Return list of valid codes"""
            return [_.name for _ in cls]

    @property
    def _doc_key(self) -> str:
        return "markdown类型"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments passed.
        :param kwargs: Keyword arguments passed.
        :return:
        """
        check_args(*args, maximum=self._MAX_LENGTH)

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to Markdown format data.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted data.
        """
        result = ({
            "msgtype": "markdown",
            "markdown": {
                "content": args[0].strip()
            }
        },)
        return result, kwargs

    @classmethod
    def _color(cls, raw: str, color: Union[str, _Color]) -> str:
        """
        Convert normal string to colorful string.
        :param raw: Raw string.
        :param color: Specify _color. Support: green | gray | orange
        :return: Colorized string.
        """
        if isinstance(color, str):
            try:
                color = cls._Color(color.lower())
            except ValueError as error:
                valid_colors = cls._Color.get_valid_colors()
                raise ValueError(
                    f"Invalid color '{color}'. Valid options: {valid_colors}"
                ) from error
        result = f'<font color="{color.name.lower()}">{raw}</font>'
        return result

    @classmethod
    def green(cls, text: str) -> str:
        """
        Return a green text string.
        :param text: Text to be converted.
        :return:
        """
        return cls._color(text, cls._Color.INFO)

    @classmethod
    def gray(cls, text: str) -> str:
        """
        Return a gray text string.
        :param text: Text to be converted.
        :return:
        """
        return cls._color(text, cls._Color.COMMENT)

    @classmethod
    def orange(cls, text: str) -> str:
        """
        Return an orange text string.
        :param text: Text to be converted.
        :return:
        """
        return cls._color(text, cls._Color.WARNING)


class MarkdownBotV2(AbstractBot):
    """Markdown V2 type message Wecom Group Bot"""

    _MAX_LENGTH: int = 4096

    @property
    def _doc_key(self) -> str:
        return "markdown-v2类型"

    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments passed.
        :param kwargs: Keyword arguments passed.
        :return:
        """
        check_args(*args, maximum=self._MAX_LENGTH)

    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to Markdown format data.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted data.
        """
        result = ({
            "msgtype": "markdown_v2",
            "markdown_v2": {
                "content": args[0].strip()
            }
        },)
        return result, kwargs

    @classmethod
    def list2table(cls, table: List[List[str]]) -> str:
        """
        Convert list of list to Markdown table string.
        :param table: List of lists to convert.
        :return: Markdown table string.
        """
        if not table or len(table) < 2:
            return ""

        # Parse headers and rows
        headers, rows = table[0], table[1:]

        # Generate header line
        header_line = "| " + " | ".join(headers) + " |"

        # Generate split line
        separator = "|"
        for i in range(len(headers)):
            if i == 0:
                separator += " :----- |"
            elif i == len(headers) - 1:
                separator += " -------: |"
            else:
                separator += " :----: |"

        # Generate data rows
        data_lines = []
        for row in rows:
            data_lines.append("| " + " | ".join(row) + " |")

        # Merge all rows
        markdown_table = "\n".join([header_line, separator] + data_lines)
        return markdown_table



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Voice type message sender

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/30 16:49
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from pathlib import Path
from logging import getLogger

from ._abstract import ConvertedData, AbstractBot
from .._deco import verify_file

logger = getLogger(__name__)


class VoiceBot(AbstractBot):
    """Voice type message Wecom Group Bot"""

    @property
    def _doc_key(self) -> str:
        return "语音类型"

    @verify_file
    def _verify_arguments(self, *args, **kwargs) -> None:
        """
        Verify the arguments passed.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return:
        """
        file_path = Path(kwargs["file_path"])
        test = kwargs.get("test")
        # Check format, only support: `.amr`
        if file_path.suffix.lower() != ".amr" or test == "wrong_format_voice":
            raise ValueError("Just support voice type: amr")

    # pylint:disable=unused-argument
    def _convert_arguments(self, *args, **kwargs) -> ConvertedData:
        """
        Convert the message to Voice format.
        :param args: Positional arguments.
        :param kwargs: Other keyword arguments.
        :return: Converted message.
        """
        file_path = kwargs["file_path"]
        # Check voice size, only smaller than `2M` and large than `5B`
        with open(file_path, "rb") as _:
            content = _.read()
        size_range = 5 < len(content) < 2 * pow(1024, 2)
        test = kwargs.get("test")
        if not size_range or test == "oversize_voice":
            raise ValueError("The voice size is out of range: 5B < SIZE < 2M")
        # Check voice duration, only less than `60s`
        try:
            # pylint: disable=import-outside-toplevel
            from pydub import AudioSegment
            audio = AudioSegment.from_file(file_path, format="amr")
            print("Checking the duration of the voice file...")
        except ImportError as error:  # pragma: no cover
            logger.debug("Raised error: %s", error)
            logger.warning("Full feature requires `pydub` to be installed.")
            logger.warning(
                'Re-install this package using `pip install "pywgb[all]"` will fix this warning.'
            )
            print("Required package `pydub` not found. Skip check duration...")
            audio = []
        if len(audio) / 1000 > 60 or test == "overlong_voice":
            raise ValueError("The voice duration is longer than 60s")
        media_id = self.upload(file_path)
        result = {"msgtype": "voice", "voice": {"media_id": media_id}}
        return (result,), kwargs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Template Card classes

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/4 17:30
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from functools import partial
from typing import List, Callable, Dict

from jmespath import search

TemplateCardKeys: List[str] = [
    "source",
    "main_title",
    "quote_area",
    "horizontal_content_list",
    "jump_list",
    "card_action",
]
TemplateCardRequirements: Dict[str, Callable] = {
    "When `quote_area.type` is 1, the `quote_area.url` must be existed":
        partial(
            search, """
                quote_area.type == `1` &&
                (quote_area.url == null || quote_area.url == '')
            """),
    "When `quote_area.type` is 2, the `quote_area.appid` must be existed":
        partial(
            search, """
                quote_area.type == `2` &&
                (quote_area.appid == null || quote_area.appid == '')
            """),
    "The `horizontal_content_list.keyname` must be existed":
        partial(
            search, """
                length(
                    (horizontal_content_list || `[]`)[?
                        keyname == null || keyname == ''
                    ]
                ) != `0`
            """),
    "When `horizontal_content_list.type` is 1, "
    "the `horizontal_content_list.url` must be existed":
        partial(
            search, """
                length(
                    (horizontal_content_list || `[]`)[?
                        type == `1` && (url == null || url == '')
                    ]
                ) != `0`
            """),
    "When `horizontal_content_list.type` is 2, "
    "the `horizontal_content_list.media_id` must be existed":
        partial(
            search, """
                length(
                    (horizontal_content_list || `[]`)[?
                        type == `2` && (media_id == null || media_id == '')
                    ]
                ) != `0`
            """),
    "When `horizontal_content_list.type` is 3, "
    "the `horizontal_content_list.userid` must be existed":
        partial(
            search, """
                length(
                    (horizontal_content_list || `[]`)[?
                        type == `3` && (userid == null || userid == '')
                    ]
                ) != `0`
            """),
    "The `jump_list.title` must be existed":
        partial(
            search, """
                length(
                    (jump_list || `[]`)[?
                        title == null || title == ''
                    ]
                ) != `0`
            """),
    "When `jump_list.type` is 1, the `jump_list.url` must be existed":
        partial(
            search, """
                length(
                    (jump_list || `[]`)[?
                        type == `1` && (url == null || url == '')
                    ]
                ) != `0`
            """),
    "When `jump_list.type` is 2, the `jump_list.appid` must be existed":
        partial(
            search, """
                length(
                    (jump_list || `[]`)[?
                        type == `2` && (appid == null || appid == '')
                    ]
                ) != `0`
            """),
    "The `card_action` and `card_action.type` must be existed":
        partial(
            search, """
                card_action == null ||
                card_action.type == null ||
                card_action.type == ''
            """),
    "When `card_action.type` is 1, the `card_action.url` must be existed":
        partial(
            search, """
                card_action.type == `1` &&
                (card_action.url == null || card_action.url == '')
            """),
    "When `card_action.type` is 2, the `card_action.appid` must be existed":
        partial(
            search, """
                card_action.type == `2` &&
                (card_action.appid == null || card_action.appid == '')
            """),
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Test error module

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/5/27 14:58
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from logging import DEBUG, basicConfig
from os import getenv
from pathlib import Path

from dotenv import load_dotenv
from pytest import raises
from yaml import safe_load

# pylint: disable=import-error
from src.pywgb.bot import TextBot, MarkdownBot, NewsBot
from src.pywgb.bot import ImageBot, VoiceBot, FileBot
from src.pywgb.bot import TextCardBot, NewsCardBot
from tests.test_main import env_file, errors_file, TEST_VALID_ARTICLES

basicConfig(level=DEBUG, format="%(levelname)s %(name)s %(lineno)d %(message)s")
load_dotenv(env_file, override=True)


def test_overheat() -> None:
    """
    Test overheat function
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    bot.send("This message was delayed by overheat", test="overheat")


def test_request_exception() -> None:
    """
    Test request exception
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    with raises(ConnectionRefusedError) as exception_info:
        bot.send("This message WON'T be sent, cause by request error",
                 test="request_error")
    assert "Unable to initiate API request correctly" in str(
        exception_info.value)
    with raises(IOError) as exception_info:
        bot.send("This message WON'T be sent, cause by API error",
                 test="api_error")
    assert "Request failed, please refer to the official manual" in str(
        exception_info.value)


def test_verify_text_error() -> None:
    """
    Test TEXT verification error.
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    with raises(ValueError) as error:
        bot.send()
    assert "The msg parameter is required" in str(error.value)
    with raises(ValueError) as error:
        bot.send("")
    assert "Can't send empty message" in str(error.value)
    with raises(ValueError) as error:
        bot.send("1" * 2049)
    assert "The msg parameter is too long" in str(error.value)
    err_msg = "parameter should be a list of strings"
    tests = [
        [123],
        (123,),
    ]
    for test in tests:
        with raises(ValueError) as error:
            bot.send("This message won't be sent, cause verify error.",
                     mentioned_mobile_list=test)
        assert err_msg in str(error.value)


def test_verify_markdown_error() -> None:
    """
    Test MARKDOWN verification error.
    :return:
    """
    bot = MarkdownBot(getenv("VALID_KEY"))
    with raises(ValueError) as error:
        bot.send()
    assert "The msg parameter is required" in str(error.value)
    with raises(ValueError) as error:
        bot.send("")
    assert "Can't send empty message" in str(error.value)
    with raises(ValueError) as error:
        # pylint: disable=protected-access
        bot._color("This will raise an exception", "red")
    assert "Invalid color" in str(error.value)


def test_verify_news_error() -> None:
    """
    Test News verification error.
    :return:
    """
    bot = NewsBot(getenv("VALID_KEY"))
    # Test empty articles
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The articles parameter is required" in str(exception_info.value)
    with raises(ValueError) as exception_info:
        bot.send(articles=[])
    assert "The articles parameter is empty" in str(exception_info.value)
    # Test oversize articles
    with raises(ValueError) as exception_info:
        articles = [TEST_VALID_ARTICLES[0] for _ in range(9)]
        bot.send(articles=articles)
    assert "Too many articles." in str(exception_info.value)
    # Test data error and parameter error
    tests = {
        "article_data_error": "data is not a dict",
        "article_parameter_error": "lack required parameter",
    }
    for code, msg in tests.items():
        with raises(ValueError) as exception_info:
            bot.send(articles=TEST_VALID_ARTICLES, test=code)
        assert msg in str(exception_info.value)


def test_verify_image_error() -> None:
    """
    Test image verification error.
    :return:
    """
    bot = ImageBot(getenv("VALID_KEY"))
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The file_path parameter is required" in str(exception_info.value)
    file = Path(__file__).with_name("test.png")
    mapper = {
        "wrong_format_image": "Just support image type: jpg or png",
        "oversize_image": "The image is too large, more than 2M",
    }
    for code, msg in mapper.items():
        with raises(ValueError) as exception_info:
            bot.send(file_path=file, test=code)
        assert msg in str(exception_info.value)


def test_verify_file_error() -> None:
    """
    Test verify file error.
    :return:
    """
    bot = FileBot(getenv("VALID_KEY"))
    # Test lack file
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The file_path parameter is required" in str(exception_info.value)
    # Test empty file
    with raises(ValueError) as exception_info:
        bot.send(file_path=None)
    assert "The file_path parameter is required" in str(exception_info.value)
    # Test invalid file
    with raises(ValueError) as exception_info:
        bot.send(file_path="234234234.txt")
    assert "The file_path parameter not exists" in str(exception_info.value)
    # Test oversize file
    file = Path(__file__).with_name("test.png")
    with raises(ValueError) as exception_info:
        bot.send(file_path=file, test="oversize_file")
    assert "The file size is out of range" in str(exception_info.value)


def test_verify_voice_error() -> None:
    """
    Test voice verification error.
    :return:
    """
    bot = VoiceBot(getenv("VALID_KEY"))
    # Test lack file
    with raises(ValueError) as exception_info:
        bot.send()
    assert "The file_path parameter is required" in str(exception_info.value)
    # Test invalid file format
    file = Path(__file__).with_name("test.png")
    with raises(ValueError) as exception_info:
        bot.send(file_path=file, test="wrong_format_voice")
    assert "Just support voice type: amr" in str(exception_info.value)
    # Test overlong and oversize voice
    file = Path(__file__).with_name("test.amr")
    with raises(ValueError) as exception_info:
        bot.send(file_path=file, test="oversize_voice")
    assert "The voice size is out of range" in str(exception_info.value)
    with raises(ValueError) as exception_info:
        bot.send(file_path=file, test="overlong_voice")
    assert "The voice duration is longer than 60s" in str(exception_info.value)


def test_verify_text_card_error() -> None:
    """
    Test Text Card verification error.
    :return:
    """
    bot = TextCardBot(getenv("VALID_KEY"))
    with open(errors_file, "r", encoding="utf-8") as _:
        tests = safe_load(_)
    for err_msg, kwargs in tests["text"].items():
        with raises(ValueError) as error:
            bot.send(**kwargs)
        assert err_msg in str(error.value)


def test_verify_news_card_error() -> None:
    """
    Test News Card verification error.
    :return:
    """
    bot = NewsCardBot(getenv("VALID_KEY"))
    with open(errors_file, "r", encoding="utf-8") as _:
        tests = safe_load(_)
    for err_msg, kwargs in tests["news"].items():
        with raises(ValueError) as error:
            bot.send(**kwargs)
        assert err_msg in str(error.value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Store successful test units.

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/3 10:41
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""

from logging import DEBUG, basicConfig
from os import getenv
from pathlib import Path
from random import randint
from urllib.parse import urlparse, unquote

from dotenv import load_dotenv
from pytest import raises

# pylint: disable=import-error
from src.pywgb.bot import TextBot, MarkdownBot, NewsBot
from src.pywgb.bot import ImageBot, VoiceBot, FileBot
from src.pywgb.bot import TextCardBot, NewsCardBot, MarkdownBotV2
from tests.test_main import VALID_KEY, env_file, TEST_VALID_MARKDOWN_V2
from tests.test_main import TEST_VALID_MARKDOWN
from tests.test_main import TEST_VALID_ARTICLES
from tests.test_main import TEST_VALID_TEXT_CARD
from tests.test_main import TEST_VALID_NEWS_CARD

basicConfig(level=DEBUG, format="%(levelname)s %(name)s %(lineno)d %(message)s")
load_dotenv(env_file, override=True)


def test_text_initial() -> None:
    """
    Test TextBot initialisation.
    :return:
    """
    valid_url = getenv("VALID_URL")
    print()
    print("Check valid key:", VALID_KEY)
    print("Check valid url:", valid_url)
    # Verify valid key and url
    bot = TextBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key
    assert f"TextBot({VALID_KEY})" == str(bot)
    assert valid_url.split("=")[-1] == TextBot(valid_url).key
    # Verify invalid key and url
    invalids = {
        getenv("INVALID_KEY"): "Invalid key format",
        getenv("INVALID_URL"): "Invalid key format",
        None: "Key is required",
    }
    for code, msg in invalids.items():
        with raises(ValueError) as exception_info:
            TextBot(code)
        assert msg in str(exception_info.value)


def test_markdown_initial() -> None:
    """
    Test MarkdownBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = MarkdownBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_markdown_v2_initial() -> None:
    """
    Test MarkdownBotV2 initialisation.
    :return:
    """
    # Verify valid key and url
    bot = MarkdownBotV2(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_image_initial() -> None:
    """
    Test ImageBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = ImageBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_news_initial() -> None:
    """
    Test NewsBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = NewsBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


# pylint: disable=protected-access
def test_file_initial() -> None:
    """
    Test NewsBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = FileBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key
    assert VALID_KEY == bot.key
    uploader = bot._uploader
    assert urlparse(unquote(uploader.doc)).fragment == uploader._doc_key
    assert VALID_KEY == uploader.key


def test_voice_initial() -> None:
    """
    Test VoiceBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = VoiceBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_text_card_initial() -> None:
    """
    Test TextCardBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = TextCardBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_news_card_initial() -> None:
    """
    Test NewsCardBot initialisation.
    :return:
    """
    # Verify valid key and url
    bot = NewsCardBot(VALID_KEY)
    assert urlparse(unquote(bot.doc)).fragment == bot._doc_key  # pylint: disable=protected-access
    assert VALID_KEY == bot.key


def test_basic_send() -> None:
    """
    Test basic send.
    :return:
    """
    bot = TextBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(f"This is a test TEXT message: {randint(1, 100)}")
    print(result)
    assert result["errcode"] == 0
    bot = MarkdownBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(TEST_VALID_MARKDOWN)
    print(result)
    assert result["errcode"] == 0
    bot = MarkdownBotV2(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(TEST_VALID_MARKDOWN_V2)
    print(result)
    assert result["errcode"] == 0
    bot = ImageBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(file_path=Path(__file__).with_name("test.png"))
    print(result)
    assert result["errcode"] == 0
    bot = NewsBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(articles=TEST_VALID_ARTICLES)
    print(result)
    assert result["errcode"] == 0
    bot = FileBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(file_path=Path(__file__).with_name("test.png"))
    print(result)
    assert result["errcode"] == 0
    bot = VoiceBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(file_path=Path(__file__).with_name("test.amr"))
    print(result)
    assert result["errcode"] == 0


def test_advanced_send() -> None:
    """
    Test advanced send function
    :return:
    """
    bot = TextCardBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(**TEST_VALID_TEXT_CARD)
    print(result)
    assert result["errcode"] == 0
    bot = NewsCardBot(getenv("VALID_KEY"))
    print(bot)
    result = bot.send(**TEST_VALID_NEWS_CARD)
    print(result)
    assert result["errcode"] == 0


def test_upload() -> None:
    """
    Test upload function
    :return:
    """
    file = Path(__file__).with_name("test.png")
    bot = TextBot(VALID_KEY)
    result = bot.upload(file)
    print(result)
    assert result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Smart bot

- Author: Rex Zhou <879582094@qq.com>
- Created Time: 2025/6/5 16:28
- Copyright: Copyright © 2025 Rex Zhou. All rights reserved.
"""
from pathlib import Path
from re import compile as re_compile, MULTILINE
from typing import Dict, List, Type, TypeVar, Union, Callable

from ._abstract import AbstractBot, FilePathLike
from .file import FileBot
from .image import ImageBot
from .markdown import MarkdownBot, MarkdownBotV2
from .news import NewsBot
from .template_card.news import NewsCardBot
from .template_card.text import TextCardBot
from .text import TextBot
from .voice import VoiceBot
from .._deco import verify_file

# pylint: disable=protected-access
_Colors: List[str] = [_.lower() for _ in MarkdownBot._Color.get_valid_codes()]
_BotT = TypeVar('_BotT', bound='AbstractBot')


class SmartBot(AbstractBot):
    """Smart Wecom Group Bot"""

    _MD_COMMON_REGEXES: List[str] = [
        r'^#{1,6}\s+.+$',  # Title
        r'\*\*.+\*\*',  # Bold
        r'`[^`]+`',  # Inner line code
        r'\[[^\]]+\]\([^\)]+\)',  # Link
        r'^>\s+.+$',  # Reference
    ]
    _MD_REGEXES: List[str] = [
        rf'<font color="({"|".join(_Colors)})">[^<]+</font>',  # Color
    ]
    _MDv2_REGEXES: List[str] = [
        *_MD_COMMON_REGEXES,
        r'\*.+\*',  # Italics
        r'^(\s*[-*+]\s+.+(\n\s{2,}\S.*)*)+$',  # Unordered list
        r'^\d+\.\s+.+$',  # Ordered List
        r'!\[[^\]]+\]\([^\)]+\)',  # Picture
        r'^(>+) .+$',  # References (multi-level support)
        r'^-{3,}$',  # Dividing line
        r'^```[\s\S]+?```$',  # Code block
        r'^\|.+\|(\n\|?[-:]+[-|: ]+)+\n(\|.+\|)+$',  # Table
    ]

    @property
    def _doc_key(self) -> str:
        return "如何使用群机器人"

    def _verify_markdown(
            self,
            string: str) -> Union[Type[MarkdownBot], Type[MarkdownBotV2], bool]:
        r"""
        Verify whether the string is Markdown format.

        For v1 version:
        - Title 1 - 6:                  r'^#{1,6}\s+.+$'
        - Bold:                         r'\*\*.+\*\*'
        - Link:                         r'\[[^\]]+\]\([^\)]+\)'
        - Code:                         r'`[^`]+`'
        - Reference:                    r'^>\s+.+$'
        - Color (** Uniq feature **):   r'<font color="(info|comment|warning)">[^<]+</font>'

        For v2 version:
        - Title 1 - 6:          r'^#{1,6}\s+.+$'
        - Bold:                 r'\*\*.+\*\*'
        - Link:                 r'\[[^\]]+\]\([^\)]+\)'
        - Code:                 r'`[^`]+`'
        - Italics:              r'\*.+\*'
        - Unordered list:       r'^(\s*[-*+]\s+.+(\n\s{2,}\S.*)*)+$'
        - Ordered List:         r'^\d+\.\s+.+$'
        - Picture:              r'!\[[^\]]+\]\([^\)]+\)'
        - References (multi):   r'^(>+) .+$'
        - Dividing line:        r'^-{3,}$'
        - Code block:           r'^```[\s\S]+?```$'
        - Table:                r'^\|.+\|(\n\|?[-:]+[-|: ]+)+\n(\|.+\|)+$'

        :param string: Raw string.
        :return: Whether the string is Markdown format.
        """
        # Top priority to detect the uniq feature for v1.
        regex = '|'.join(f'(?:{reg})' for reg in self._MD_REGEXES)
        regex = re_compile(r'(' + regex + r')', MULTILINE)
        if regex.search(string):
            return MarkdownBot
        # Try to use v2 to match content
        regex = '|'.join(f'(?:{reg})' for reg in self._MDv2_REGEXES)
        regex = re_compile(r'(' + regex + r')', MULTILINE)
        if regex.search(string):
            return MarkdownBotV2
        # Return `False` when can't match content
        return False

    def _guess_message_bot(self, *args, **kwargs) -> Type[_BotT]:
        """
        Guess whether the bot type is Text or Markdown.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: Subclass of AbstractBot.
        """
        if "mentioned_list" in kwargs or "mentioned_mobile_list" in kwargs:
            return TextBot
        if bot := self._verify_markdown(args[0]):
            return bot
        return TextBot

    @verify_file
    def _guess_file_bot(self, file_path: FilePathLike) -> Type[_BotT]:
        """
        Guess whether the bot type is File or Image or Voice.
        :param file_path: File path.
        :return: Subclass of AbstractBot.
        """
        suffix = Path(file_path).suffix
        if suffix == ".amr":
            bot = VoiceBot
        elif suffix in (".jpg", ".png"):
            bot = ImageBot
        else:
            bot = FileBot
        return bot

    def _verify_arguments(self, *args, **kwargs) -> Type[_BotT]:
        r"""
        Intelligent selection bot, follow the following rules:

        1. Provide `args`, which MUST be either Text or Markdown;

        There are 2 situations:

            Section A:
                Also provide `mentioned_list` or `mentioned_mobile_list` from kwargs;
                This MUST be Text.

            Section B:
                Distinguish using regex to verify whether the `msg` is Markdown format;

        2. Provide `file_path` from kwargs, which MUST be File / Image / Voice;
        Distinguish using file suffix.

            - Voice: MUST be `.amr` file.
            - Image: Either `.png` or `.jpg`.
            - File: Other suffixes.

        3. Provide `articles` from kwargs, which MUST be News;

        4. Complex kwargs, which MUST be either TextCard or NewsCard;
        Distinguish using bot's own required parameters.

            - NewsCard: MUST have `card_image` in kwargs.
            - TextCard: Otherwise default bot.

        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :return: Subclass of AbstractBot.
        """
        if args:
            return self._guess_message_bot(*args, **kwargs)
        if "file_path" in kwargs and kwargs["file_path"]:
            return self._guess_file_bot(kwargs["file_path"])
        if "articles" in kwargs and kwargs["articles"]:
            return NewsBot
        if "card_image" in kwargs:
            return NewsCardBot
        return TextCardBot

    def _convert_arguments(self, *args, **kwargs) -> AbstractBot:
        bot_type: Type[_BotT] = args[0]
        return bot_type(self.key)

    def send(self,
             msg: str = None,
             /,
             articles: List[Dict[str, str]] = None,
             file_path: FilePathLike = None,
             **kwargs) -> dict:
        """
        Automatic select correct type of bot, then send the content.
        :param msg: Message to send. Valid for Text or Markdown.
        :param articles: Articles to send. Valid for News.
        :param file_path: File to send. Valid for Image | Voice | File.
        :param kwargs: Other keyword arguments. Valid for Text(mention) | TextCard | NewsCard.
        :return:
        """
        if msg is not None:
            bot_type = self._verify_arguments(msg, **kwargs)
            bot = self._convert_arguments(bot_type)
        else:
            bot_type = self._verify_arguments(articles=articles,
                                              file_path=file_path,
                                              **kwargs)
            bot = self._convert_arguments(bot_type)
        result = bot.send(msg, articles=articles, file_path=file_path, **kwargs)
        return result

    # pylint: disable=too-few-public-methods
    class _MarkdownFeatureProxy:
        """Proxy that routes method calls to appropriate Markdown type."""

        @property
        def green(self) -> Callable:
            """
            Return v1 feature.
            :return:
            """
            return MarkdownBot.green

        @property
        def gray(self) -> Callable:
            """
            Return v1 feature.
            :return:
            """
            return MarkdownBot.gray

        @property
        def orange(self) -> Callable:
            """
            Return v1 feature.
            :return:
            """
            return MarkdownBot.orange

        @property
        def list2table(self) -> Callable:
            """
            Return v2 feature.
            :return:
            """
            return MarkdownBotV2.list2table

    markdown_feature = _MarkdownFeatureProxy()
